/*----------------------------------------------------------------------- 

File Name: ArrayFactories.c

<lalVerbatim file="ArrayFactoriesCV">
Revision: $Id: ArrayFactories.m4,v 1.5 2003/09/29 00:33:24 dwchin Exp $
</lalVerbatim>

-------------------------------------------------------------------------*/

/* <lalLaTeX>

\subsection{Module \texttt{ArrayFactories.c}}
\label{ss:ArrayFactories.c}

Create/destroy $\langle\mbox{datatype}\rangle$Array objects. 

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{ArrayFactoriesD}
\idx{LALZCreateArray()}
\idx{LALCCreateArray()}
\idx{LALDCreateArray()}
\idx{LALSCreateArray()}
\idx{LALI2CreateArray()}
\idx{LALI4CreateArray()}
\idx{LALI8CreateArray()}
\idx{LALU2CreateArray()}
\idx{LALU4CreateArray()}
\idx{LALU8CreateArray()}
\idx{LALCreateArray()}
\idx{LALZDestroyArray()}
\idx{LALCDestroyArray()}
\idx{LALDDestroyArray()}
\idx{LALSDestroyArray()}
\idx{LALI2DestroyArray()}
\idx{LALI4DestroyArray()}
\idx{LALI8DestroyArray()}
\idx{LALU2DestroyArray()}
\idx{LALU4DestroyArray()}
\idx{LALU8DestroyArray()}
\idx{LALDestroyArray()}

\subsubsection*{Description}

The \texttt{CreateArray} family of functions create a
$\langle\mbox{datatype}\rangle$\texttt{Array} of the appropriate dimensions.

The \texttt{DestroyArray} family of functions return the storage allocated by
the \texttt{CreateArray} functions to the system.

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
LALMalloc()
LALFree()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{ArrayFactoriesCV}}

</lalLaTeX> */

#include <string.h>
#include "LALStdlib.h"
#include "AVFactories.h"

/* <lalVerbatim file="ArrayFactoriesNRCSID"> */
NRCSID( ARRAYFACTORIESC, "$Id: ArrayFactories.m4,v 1.5 2003/09/29 00:33:24 dwchin Exp $" );
/* </lalVerbatim> */
















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALZCreateArray ( LALStatus *status, COMPLEX16Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALZCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( COMPLEX16Array * ) LALMalloc ( sizeof( COMPLEX16Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( COMPLEX16 * ) LALMalloc( arrayDataSize*sizeof( COMPLEX16 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALZResizeArray ( LALStatus *status, COMPLEX16Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  COMPLEX16 * p; /* temporary pointer */

  INITSTATUS (status, "LALZResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALZCreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALZDestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALZDestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( COMPLEX16 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALZDestroyArray ( LALStatus *status, COMPLEX16Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALZDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALCCreateArray ( LALStatus *status, COMPLEX8Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALCCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( COMPLEX8Array * ) LALMalloc ( sizeof( COMPLEX8Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( COMPLEX8 * ) LALMalloc( arrayDataSize*sizeof( COMPLEX8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALCResizeArray ( LALStatus *status, COMPLEX8Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  COMPLEX8 * p; /* temporary pointer */

  INITSTATUS (status, "LALCResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALCCreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALCDestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALCDestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( COMPLEX8 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALCDestroyArray ( LALStatus *status, COMPLEX8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALCDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALDCreateArray ( LALStatus *status, REAL8Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALDCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL8Array * ) LALMalloc ( sizeof( REAL8Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL8 * ) LALMalloc( arrayDataSize*sizeof( REAL8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALDResizeArray ( LALStatus *status, REAL8Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  REAL8 * p; /* temporary pointer */

  INITSTATUS (status, "LALDResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALDCreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALDDestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALDDestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( REAL8 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALDDestroyArray ( LALStatus *status, REAL8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALDDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALSCreateArray ( LALStatus *status, REAL4Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALSCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL4Array * ) LALMalloc ( sizeof( REAL4Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL4 * ) LALMalloc( arrayDataSize*sizeof( REAL4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALSResizeArray ( LALStatus *status, REAL4Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  REAL4 * p; /* temporary pointer */

  INITSTATUS (status, "LALSResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALSCreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALSDestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALSDestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( REAL4 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALSDestroyArray ( LALStatus *status, REAL4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALSDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI2CreateArray ( LALStatus *status, INT2Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALI2CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT2Array * ) LALMalloc ( sizeof( INT2Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT2 * ) LALMalloc( arrayDataSize*sizeof( INT2 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI2ResizeArray ( LALStatus *status, INT2Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  INT2 * p; /* temporary pointer */

  INITSTATUS (status, "LALI2ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALI2CreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALI2DestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALI2DestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( INT2 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI2DestroyArray ( LALStatus *status, INT2Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI2DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI4CreateArray ( LALStatus *status, INT4Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALI4CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT4Array * ) LALMalloc ( sizeof( INT4Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT4 * ) LALMalloc( arrayDataSize*sizeof( INT4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI4ResizeArray ( LALStatus *status, INT4Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  INT4 * p; /* temporary pointer */

  INITSTATUS (status, "LALI4ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALI4CreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALI4DestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALI4DestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( INT4 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI4DestroyArray ( LALStatus *status, INT4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI4DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI8CreateArray ( LALStatus *status, INT8Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALI8CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( INT8Array * ) LALMalloc ( sizeof( INT8Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( INT8 * ) LALMalloc( arrayDataSize*sizeof( INT8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI8ResizeArray ( LALStatus *status, INT8Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  INT8 * p; /* temporary pointer */

  INITSTATUS (status, "LALI8ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALI8CreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALI8DestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALI8DestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( INT8 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALI8DestroyArray ( LALStatus *status, INT8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALI8DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU2CreateArray ( LALStatus *status, UINT2Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALU2CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT2Array * ) LALMalloc ( sizeof( UINT2Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT2 * ) LALMalloc( arrayDataSize*sizeof( UINT2 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU2ResizeArray ( LALStatus *status, UINT2Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  UINT2 * p; /* temporary pointer */

  INITSTATUS (status, "LALU2ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALU2CreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALU2DestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALU2DestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( UINT2 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU2DestroyArray ( LALStatus *status, UINT2Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU2DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU4CreateArray ( LALStatus *status, UINT4Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALU4CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT4Array * ) LALMalloc ( sizeof( UINT4Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT4 * ) LALMalloc( arrayDataSize*sizeof( UINT4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU4ResizeArray ( LALStatus *status, UINT4Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  UINT4 * p; /* temporary pointer */

  INITSTATUS (status, "LALU4ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALU4CreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALU4DestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALU4DestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( UINT4 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU4DestroyArray ( LALStatus *status, UINT4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU4DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU8CreateArray ( LALStatus *status, UINT8Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALU8CreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( UINT8Array * ) LALMalloc ( sizeof( UINT8Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( UINT8 * ) LALMalloc( arrayDataSize*sizeof( UINT8 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU8ResizeArray ( LALStatus *status, UINT8Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  UINT8 * p; /* temporary pointer */

  INITSTATUS (status, "LALU8ResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALU8CreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALU8DestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALU8DestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( UINT8 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALU8DestroyArray ( LALStatus *status, UINT8Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALU8DestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALCreateArray ( LALStatus *status, REAL4Array **array, UINT4Vector *dimLength ) 
{ /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;

  INITSTATUS (status, "LALCreateArray", ARRAYFACTORIESC);   
  ATTATCHSTATUSPTR (status);

  /* make sure arguments are sane */

  ASSERT (array,             status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (!*array,           status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT (dimLength,         status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (dimLength->length, status,
          AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  numDims = dimLength->length;

  /* loop over dimensions to compute total size of array data */

  for (dim = 0; dim < numDims; ++dim)
  {
    arrayDataSize *= dimLength->data[dim];
  }

  ASSERT (arrayDataSize, status, AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

  /* allocate memory for structure */

  *array = ( REAL4Array * ) LALMalloc ( sizeof( REAL4Array ) );
  if ( NULL == *array )
  {
    ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
  }

  (*array)->dimLength = NULL;
  (*array)->data      = NULL;

  /* allocate dimLength field and copy information there */

  LALU4CreateVector (status->statusPtr, &(*array)->dimLength, numDims);
  if (status->statusPtr->statusCode)
  {
    LALFree (*array);
    *array = NULL;
    RETURN (status);   /* this returns a recursive error status code (-1) */
  }
  memcpy ((*array)->dimLength->data, dimLength->data, numDims*sizeof(UINT4));
  
  /* allocate storage */

  (*array)->data = ( REAL4 * ) LALMalloc( arrayDataSize*sizeof( REAL4 ) );

  if (!(*array)->data)
  {
    /* try to free memory                                               */
    /* this should ALWAYS work so we can use the CHECKSTATUSPTR() macro */
    LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
    CHECKSTATUSPTR (status);  /* if this fails, there is a memory leak  */

    LALFree (*array);
    *array = NULL;
    ABORT (status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC);
  }

  DETATCHSTATUSPTR (status);
  RETURN (status);
}



















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALResizeArray ( LALStatus *status, REAL4Array **array, UINT4Vector *dimLength )
{  /* </lalVerbatim> */
  UINT4 arrayDataSize = 1;
  UINT4 numDims;
  UINT4 dim;
  REAL4 * p; /* temporary pointer */

  INITSTATUS (status, "LALResizeArray", ARRAYFACTORIESC);

  ASSERT ( array != NULL, status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR );

  if ( (*array) == NULL )
    {
      LALCreateArray ( status, array, dimLength );
    }
  else if ( dimLength == NULL )
    {
      LALDestroyArray ( status, array );
    }
  else
    {
      ASSERT (dimLength->data,   status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
      ASSERT (dimLength->length, status,
              AVFACTORIESH_ELENGTH, AVFACTORIESH_MSGELENGTH);

      numDims = dimLength->length;

      /* loop over dimensions to compute total size of array data */
      for (dim = 0; dim < numDims; ++dim)
        arrayDataSize *= dimLength->data[dim];
        
      ASSERT (arrayDataSize > 0, status, AVFACTORIESH_ELENGTH, 
              AVFACTORIESH_MSGELENGTH);

      if (arrayDataSize == 0)
        {
          LALDestroyArray ( status, array );
        }
      else
        {
          /* copy over the dimension data */
          (*array)->dimLength->length = dimLength->length;
          memcpy((*array)->dimLength->data, dimLength->data, 
                 numDims*sizeof(UINT4));

          /* reallocate memory for array data */
          p = LALRealloc((*array)->data, arrayDataSize*sizeof( REAL4 ));

          if (p == NULL)
            {
              LALFree( (*array)->data );
              ABORT( status, AVFACTORIESH_EMALLOC, AVFACTORIESH_MSGEMALLOC );
            }

          (*array)->data = p;
        }
    }
    
  RETURN (status);
}















/* <lalVerbatim file="ArrayFactoriesD"> */
void LALDestroyArray ( LALStatus *status, REAL4Array **array )
{ /* </lalVerbatim> */
  INITSTATUS (status, "LALDestroyArray", ARRAYFACTORIESC);	
  ATTATCHSTATUSPTR (status);
      
  ASSERT (array,          status, AVFACTORIESH_EVPTR, AVFACTORIESH_MSGEVPTR);
  ASSERT (*array,         status, AVFACTORIESH_EUPTR, AVFACTORIESH_MSGEUPTR);
  ASSERT ((*array)->data, status, AVFACTORIESH_EDPTR, AVFACTORIESH_MSGEDPTR);

  /* Free allocated storage */

  LALU4DestroyVector (status->statusPtr, &(*array)->dimLength);
  CHECKSTATUSPTR (status);

  LALFree ((*array)->data); /* free allocated data */
  LALFree (*array);	    /* free array struct itself */
  *array = NULL;	    /* make sure we don't point to freed struct */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

